SystemOrganization addCategory: #'Sudoku-Core'!

Matrix subclass: #SudokuGrid
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sudoku-Core'!

!SudokuGrid class methodsFor: 'accessing' stamp: 'mg 5/28/2014 12:59'!
boxIndexDict
"This method returns a dictionary that associate the index of a 3x3 box with the position of the top left cell of the box."
	^ Dictionary new
		at: 1 put: 1@1;
		at: 2 put: 1@4;
		at: 3 put: 1@7;
		at: 4 put: 4@1;
		at: 5 put: 4@4;
		at: 6 put: 4@7;
		at: 7 put: 7@1;
		at: 8 put: 7@4;
		at: 9 put: 7@7;
		yourself! !

!SudokuGrid class methodsFor: 'instance creation' stamp: 'mg 5/28/2014 13:14'!
getCandidatesMatrixFrom: aMatrix

	| allCandidates candidates |
	allCandidates := OrderedCollection new.
	1 to: 9 do:
		[:i | allCandidates add: i].
	
	candidates := Matrix new: 9.
	1 to: 9 do:
		[:i | 1 to: 9 do:
			[:j | ((aMatrix at: i at: j) = 0)
					ifTrue: [candidates at: i at: j put: allCandidates copy]
					ifFalse: [candidates at: i at: j put: (OrderedCollection with: (aMatrix at: i at: j))]]].

	^ candidates! !

!SudokuGrid class methodsFor: 'instance creation' stamp: 'mg 6/4/2014 11:12'!
newFrom: aMatrix

	^ self rows: aMatrix rowCount columns: aMatrix columnCount contents: aMatrix asArray deepCopy! !

!SudokuGrid class methodsFor: 'instance creation' stamp: 'mg 5/28/2014 13:14'!
newFromFile: aFileName
	| file numberList numbers |

	numberList := OrderedCollection new.
	file := MultiByteFileStream fileNamed: aFileName.
	file upToEnd asString substrings
		collect: [:row | row do: [:digit | numberList add: digit digitValue]].

	numbers := Matrix new: 9 tabulate: [:i :j | numberList at: (j \\ 10 + (i-1*9))].
	file close.
	
	^ self newFrom: numbers! !

!SudokuGrid class methodsFor: 'accessing' stamp: 'mg 5/28/2014 12:59'!
topLeftCellOfBox: index

	^ self boxIndexDict at: index! !

!SudokuGrid methodsFor: 'accessing boxes' stamp: 'mg 6/3/2014 13:59'!
atBox: index
"This method returns a new collection of all cells in the box at index."
	| box topLeftIndex |
	(index between: 1 and: nrows)
		ifFalse: [self error: 'box subscript out of range'].
	box := self contents class new.
	topLeftIndex := self class topLeftCellOfBox: index.
	0 to: 2 do:
		[:x | 0 to: 2 do:
			[:y | box add: (self at: topLeftIndex x + x at: topLeftIndex y + y)]].
	^ box! !
